@@ -1,13 +1,14 @@
 # -*- coding: utf-8 -*-
-
-__version__ = "0.4.2.3"
+__version__ = "0.4.3"
 __useragent__ = ('ldtools-%s (http://github.com/dmr/ldtools, daniel@nwebs.de)'
                  % __version__)
 
 import datetime
+import logging
 import mimetypes
 import os
 import rdflib
+import socket;
 import shutil
 import urllib2
 from rdflib.namespace import split_uri
@@ -15,10 +16,9 @@ from rdflib import compare
 from urlparse import urlparse
 from xml.sax._exceptions import SAXParseException
 
-import socket;
-socket.setdefaulttimeout(5) # HACK, TODO: find a way to set this for request
+# TODO: find a better way to set socket timeout
+socket.setdefaulttimeout(5)
 
-import logging
 logger = logging.getLogger(__name__)
 
 
@@ -140,16 +140,10 @@ class ObjectField(Field):
         return value
 
 
-
 class Backend(object):
     """ Abstract Backend to demonstrate API
     """
-    # TODO: "Backend" if it manages one file/resource?
-
-    def __init__(self, uri):
-        self.uri = uri
-
-    def GET(self):
+    def GET(self, uri):
         raise NotImplementedError
     def PUT(self):
         raise NotImplementedError
@@ -159,6 +153,7 @@ class Backend(object):
         file_name = uri.lstrip("http://").replace(".","_")\
             .replace("/","__").replace("?","___").replace("&","____")
 
+        # TODO: hacky
         folder = os.path.abspath("cache")
         if not os.path.exists(folder):
             os.mkdir(folder)
@@ -168,8 +163,16 @@ class Backend(object):
 
 class RestBackend(Backend):
 
-    def GET(self):
+    def GET(self, uri):
         """lookup URI"""
+
+        if not hasattr(self, "uri"):
+            self.uri = uri
+        else:
+            if not self.uri == uri:
+                raise Exception("You cannot pass different uris to the same "
+                                "backend")
+
         # TODO: friendly crawling: use robots.txt
         # crawling speed limitations in robots.txt.
 
@@ -214,6 +217,8 @@ class RestBackend(Backend):
         return content
 
     def PUT(self, graph):
+        assert self.uri, "GET has to be called before PUT possible"
+
         import mimetypes
         mimetypes.init()
 
@@ -222,7 +227,6 @@ class RestBackend(Backend):
 
         self.format = content_type
 
-        # TODO: maybe 'pretty-xml'?
         data = graph.serialize(format=self.format)
 
         # TODO: authentication? oauth?
@@ -241,50 +245,55 @@ class RestBackend(Backend):
         request.get_method = lambda: 'PUT'
         response = opener.open(request)
 
+        assert response.status_code in [201, 200]
+        return response.status_code
+
 
 class SingleFileBackend(Backend):
     """Manages one xml file --> Uri that the user wants to "PUT" to is not
     flexible!
     """
 
-    def __init__(self, uri, filename, format="xml"):
-        super(SingleFileBackend, self).__init__(uri)
-
-        # TODO: filename already contains ".xml" --> read from there
-        # TODO: make filename optional and calculate
+    def __init__(self, filename, format=None):
         assert os.path.exists(filename)
         self.filename = filename
-        # TODO: assert format in rdflib.parserplugins
-        self.format = format
 
-    def GET(self):
+        # TODO: assert format in rdflib.parserplugins
+        self.format = format if format else self.filename.split(".")[-1]
+
+    def GET(self, uri):
+        if not hasattr(self, "uri"):
+            self.uri = uri
+        else:
+            if not self.uri == uri:
+                raise Exception("You cannot pass different uris to the same "
+                                "backend")
+
         with open(self.filename, "r") as f:
             data = f.read()
         return data
 
     def PUT(self, graph):
+        assert self.uri, "GET has to be called before PUT possible"
+
         mimetypes.init()
 
         # 'application/rdf+xml'
         content_type = mimetypes.types_map[".%s" % self.format]
 
-        # we want to update it --> it must exist first!
-        assert os.path.exists(self.filename)
+        if os.path.exists(self.filename):
+            # File already exists. Make backup copy
+            now = datetime.datetime.strftime(datetime.datetime.utcnow(),
+                                             '%Y%m%d-%H%M%S')
+            file_extension = get_file_extension(self.filename)
+            if file_extension:
+                old_version = u"%s.%s.%s" % (self.filename.strip(file_extension),
+                                             now, file_extension)
+            else:
+                old_version = u"%s_%s" % (self.filename, now)
+            shutil.copy(self.filename, old_version)
+
         data = graph.serialize(format=self.format)
-
-        now = datetime.datetime.strftime(datetime.datetime.utcnow(),
-                                         '%Y%m%d-%H%M%S')
-        assert now
-
-        file_extension = get_file_extension(self.filename)
-        if file_extension:
-            old_version = u"%s.%s.%s" % (self.filename.strip(file_extension),
-                                         now, file_extension)
-        else:
-            old_version = u"%s_%s" % (self.filename, now)
-
-        shutil.copy(self.filename, old_version)
-
         with open(self.filename, "w") as f:
             f.write(data)
         self.old_version = old_version
@@ -297,6 +306,14 @@ class SingleFileBackend(Backend):
             delattr(self, "old_version")
 
 
+class SimpleMemoryBackend(Backend):
+    def __init__(self, data, format="xml"):
+        self.data = data
+        self.format = format
+    def GET(self, uri):
+        return self.data
+
+
 class Options(object):
 
     def __init__(self, meta, attrs):
@@ -351,32 +368,45 @@ class Model(object):
 
     def __init__(self, pk=None, **kwargs):
         self.pk = pk
-        for attrname, field in self._meta.fields.iteritems():
-            attr = kwargs.pop(attrname)
+        for attr_name, field in self._meta.fields.iteritems():
+            attr = kwargs.pop(attr_name)
             value = field.to_python(attr)
-            setattr(self, attrname, value)
+            setattr(self, attr_name, value)
         if kwargs:
+            # TODO ValueError is raised on object creation with kwargs.
+            # inconsistent: after that, no checks performed.
             raise ValueError('%s are not part of the schema for %s'
                 % (', '.join(kwargs.keys()), self.__class__.__name__))
 
     def __eq__(self, other):
         if not type(other) == type(self):
             return False
-        if not other.__dict__.keys() == self.__dict__.keys():
-            return False
-        if not all(getattr(other, key) == getattr(self, key)
-                    for key in self.__dict__.keys()):
+
+        if set(other.__dict__.keys()) != set(self.__dict__.keys()):
             return False
+
+        for key in self.__dict__.keys():
+            if (not hasattr(other, key) or
+                    getattr(self, key) != getattr(other, key)):
+                return False
+
         return True
 
 
     def __ne__(self, other):
         if type(other) != type(self):
             return True
-        if any(getattr(other, key) != getattr(self, key) \
-                   for key in self.__dict__.keys()):
+
+        if set(other.__dict__.keys()) != set(self.__dict__.keys()):
             return True
 
+        for key in self.__dict__.keys():
+            if (not hasattr(other, key) or
+                    getattr(other, key) != getattr(self, key)):
+                return True
+
+        return False
+
     def __hash__(self):
         # It is wrong to really compare the object here. This case is
         # important to work with references in set() instances for instance
@@ -420,11 +450,11 @@ class Manager(object):
 
     def filter(self, **kwargs):
         def check_if_equals_or_in_set((key, value)):
+            """helper method for loop. Convenient but maybe hacky: checks
+            if value is in attr or if iterable inside the iterable"""
             if hasattr(item, key):
                 items_value = getattr(item, key)
-                # TODO: inconsistent: can handle lists and strings!!!
-                # convenient but ugly
-                if type(items_value) in (set, list):
+                if hasattr(items_value, "__iter__"):
                     if value in items_value:
                         return True
                 else:
@@ -432,10 +462,8 @@ class Manager(object):
                         return True
             return False
 
-        # TODO: this is ugly but it works ;-)
         for item in self.all():
             if all(map(check_if_equals_or_in_set, kwargs.items())):
-                #logger.info("Found %s in %s" % (kwargs.items(), item))
                 yield item
 
     def create(self, pk, **kwargs):
@@ -454,10 +482,10 @@ class Manager(object):
 
 class ResourceManager(Manager):
 
-    def get_pk(self, uri1, uri2):
-        return uri1+uri2
+    def get_pk(self, origin_uri, uri):
+        return origin_uri+uri
 
-    def create(self, uri, origin):
+    def create(self, uri, origin, **kwargs):
         assert isinstance(origin, Origin), "Origin instance required"
 
         if uri.startswith("#"):
@@ -466,15 +494,16 @@ class ResourceManager(Manager):
             uri = rdflib.URIRef(origin.uri + uri)
 
         uri = canonalize_uri(uri)
-        pk = self.get_pk(origin.uri, uri)
+        pk = self.get_pk(origin_uri=origin.uri, uri=uri)
         if isinstance(uri, rdflib.BNode):
             obj = super(ResourceManager, self).create(pk=pk, _uri=uri,
                                                       _origin=origin)
             assert isinstance(obj._uri, rdflib.BNode)
             assert not hasattr(obj, "_has_changes")
             return obj
+
         return super(ResourceManager, self).create(pk=pk, _uri=uri,
-                                                   _origin=origin)
+                                                   _origin=origin, **kwargs)
 
     def get(self, uri, origin=None):
         """If the authoratative Origin to the Resource does not exist and no
@@ -505,7 +534,7 @@ class ResourceManager(Manager):
 
 
         assert isinstance(origin, Origin), origin
-        pk = self.get_pk(origin.uri, uri)
+        pk = self.get_pk(origin_uri=origin.uri, uri=uri)
         return super(ResourceManager, self).get(pk=pk)
         assert 0, "implement!"
 
@@ -529,7 +558,7 @@ def reverse_dict(dct):
     for k,v in dct.iteritems():
         res[v] = k
     return safe_dict(res)
-    
+
 
 def predicate2pyattr(predicate, namespacedict):
     prefix, propertyname = split_uri(predicate)
@@ -542,6 +571,7 @@ def predicate2pyattr(predicate, namespacedict):
         return predicate
     return u"%s_%s" % (namespacedict[prefix], propertyname)
 
+
 def pyattr2predicate(pyattr, namespacedict):
     # TODO: build urirefdict instead of unicodedict?
 
@@ -572,10 +602,9 @@ class Resource(Model):
     _origin = ObjectField()
     objects = ResourceManager()
 
-    def add_property(self, predicate, object):
+    def _add_property(self, predicate, object):
         assert isinstance(predicate, rdflib.URIRef),\
             "Not an URIRef: %s"%predicate
-        # TODO remove?
         assert hasattr(predicate, "n3"),\
             "property %s is not a rdflib object" % predicate
 
@@ -622,10 +651,12 @@ class Resource(Model):
             attr = getattr(self, predicate)
             #assert attr == object
 
+        # TODO: this might cause UnicodeDecodeErrors
         #logger.debug(u"Contribute_to_object %s: %s = %s"
         #    % (self, predicate.encode('utf8'), attr))
         
-        # --> TODO: force_unicode when reconverting from __dict__?
+        # TODO: introduce rdflib.Literal dict or force_unicode when
+        # reconverting from __dict__?
         assert str(predicate) in self.__dict__
 
     def __unicode__(self):
@@ -637,27 +668,21 @@ class Resource(Model):
             assert isinstance(self._origin, Origin), \
                     "%s" % (getattr(self, "_origin", None))
             str += u" [%r]" % self._origin
-        #if hasattr(self, "rdf_type"): # rdflib.RDF.type):
-        #    rdf_type = list(self.rdf_type)
-        #    #list(getattr(self, rdflib.RDF.type))
-        #    str += (u" rdf:type %s" % unicode(rdf_type[0]))
-        #    if len(rdf_type) > 1: str += u",..."
+        if hasattr(self, "rdf_type"):
+            rdf_type = list(self.rdf_type)
+            str += (u" rdf:type %s" % unicode(rdf_type[0]))
+            if len(rdf_type) > 1:
+                str += u",..."
         return str
 
-    def tripleserialize_iterator(self):
-        # TODO: self.get_property_dict() instead?
+    def _tripleserialize_iterator(self, namespace_dict):
         for property, values in self.__dict__.items():
 
             # skip internals
             if str(property).startswith("_") or property == "pk":
                 continue
 
-            # TODO: better idea how to do this?
-            # __dict__ converts rdflib.urirefs to strings -->
-            # converts back to uriref
-            namespacedict = dict(self._origin._graph.namespace_manager\
-                            .namespaces())
-            property = pyattr2predicate(property, namespacedict)
+            property = pyattr2predicate(property, namespace_dict)
 
             assert hasattr(property, "n3"), \
                 "property %s is not a rdflib object" % property
@@ -684,7 +709,7 @@ class Resource(Model):
             field = self._meta.fields.get(key)
             if field and value:
                 value = field.to_python(value)
-        elif not key.startswith("_") and not key == "pk": # TODO: --> _pk?
+        elif not key.startswith("_") and not key == "pk": # TODO: rename to_pk?
             # rdf properties here
             self._has_changes = True
         Model.__setattr__(self, key, value)
@@ -697,22 +722,29 @@ class Resource(Model):
         self.__class__.objects._storage.__delitem__(self.pk)
 
     def save(self):
+        # TODO: introduce "clean" to make sure only valid properties modified
         created = not self.pk
-        assert not created, "please use Resource.objects.create() til later"
-        #if created: # self.pk = app.db.add(model)
+        if created:
+            assert 0, ("Please use Resource.objects.create() to create Resource"
+                       "objects!")
+            #assert not self.__class__.objects.get(self.uri, self.origin)
+            #self = self.__class__.objects.create(**self.get_attributes())
 
+        # TODO: write tests for the following lines
         assert self in Resource.objects.filter(_origin=self._origin)
         assert self in Resource.objects.filter(_origin=self._origin,
             _has_changes=True)
         assert len(list(Resource.objects.filter(_origin=self._origin,
             _has_changes=True))) == 1
+
+        #values = dict((name, getattr(self, name)) for name in \
+        #        self._meta.fields.iterkeys())
+        self.update() #**values)
+
+    def update(self):
         self._origin.PUT()
         self._has_changes = False
-    #    values = dict((name, getattr(self, name)) for name in \
-    #            self._meta.fields.iterkeys())
-    #    self.update(**values)
-    #def update(self, **values):
-    #    pass
+
 
 class OriginManager(Manager):
 
@@ -724,7 +756,7 @@ class OriginManager(Manager):
     def create(self, uri, BACKEND=None):
         uri = canonalize_uri(uri)
         self.create_hook(uri)
-        backend = BACKEND if BACKEND else RestBackend(uri)
+        backend = BACKEND if BACKEND else RestBackend()
         return super(OriginManager, self).create(pk=uri, uri=uri,
                                                  backend=backend)
 
@@ -735,7 +767,7 @@ class OriginManager(Manager):
 
     def get_or_create(self, uri, **kwargs):
         assert not kwargs, ("If you intend to use 'backend' please use "
-            "Oritin.objects.create() directly")
+            "Origin.objects.create() directly")
         uri = canonalize_uri(uri)
         assert str(uri) == str(hash_to_slash_uri(uri))
         try:
@@ -745,8 +777,7 @@ class OriginManager(Manager):
 
     @catchKeyboardInterrupt
     def GET_all(self, depth=2, **kwargs):
-        """ Crawls or Re-Crawls all Origins.
-        Passes Arguments to GET"""
+        """Crawls or Re-Crawls all Origins. Passes Arguments to GET"""
         # TODO: limit crawling speed
         for _i in range(depth):
             for origin in self.all():
@@ -790,14 +821,6 @@ class Origin(Model):
                     raise NotImplementedError
         return str
 
-    def has_unsaved_changes(self):
-        if any(resource._has_changes for resource in self.get_resources()\
-               if hasattr(resource, '_has_changes')): return True
-        return False
-
-    def get_resources(self):
-        return [r for r in Origin.objects.all() if self == r._origin]
-
     def GET(self,
             GRAPH_SIZE_LIMIT=25000,
             follow_uris=None,
@@ -809,14 +832,15 @@ class Origin(Model):
         assert not self.has_unsaved_changes(), ("Please save all changes "
             "before querying again. Merging not supported yet")
 
-        # http://www.infoq.com/news/2008/04/cool-uris-rest
         if skip_urls is not None and str(self.uri) in skip_urls:
             self.add_error("Skipped")
             self.processed = True
             return
 
+        self.stats['last_processed'] = datetime.datetime.now()
+
         try:
-            data = self.backend.GET()
+            data = self.backend.GET(self.uri)
         except urllib2.HTTPError as e:
             if e.code in [
                 401,
@@ -833,7 +857,7 @@ class Origin(Model):
             self.add_error("timeout")
             return
         assert self.backend.format, "format is needed later"
-        
+
         if not data:
             self.processed = True
             return
@@ -918,6 +942,51 @@ class Origin(Model):
                 % (self.uri, self.stats['handle_graph']))
             pass
 
+    def get_resources(self):
+        return Resource.objects.filter(_origin=self)
+
+    def graph(self):
+        """Processes every Resource and Property related to 'self' and
+        creates rdflib.ConjunctiveGraph because rdflib.Graph does not allow
+        parsing plugins
+        """
+        # TODO: rdflib.graph() ?
+        graph = rdflib.graph.ConjunctiveGraph(identifier=self.uri)
+
+        if not hasattr(self, '_graph'):
+            if len(self.errors) == 0:
+                self.GET() # TODO: test for recursion?
+            else:
+                logging.error("Origin %s has Errors --> can't process .graph()"
+                    % self.uri)
+                return graph
+
+        # TODO: find a better way to do this
+        # Problems:
+        #  1) namespace bindings are not really necessary to validate
+        #     isomorphic graphs but the resulting graph is is different
+        #     if they miss
+        #  2) doesn't detect duplicate definitions of namespaces
+        namespaces = dict(self._graph.namespace_manager\
+                            .namespaces())
+        for prefix, namespace in safe_dict(namespaces).items():
+            graph.bind(prefix=prefix, namespace=namespace)
+        new_ns = dict(graph.namespace_manager.namespaces())
+
+        assert namespaces == new_ns, [(k, v) for k, v in
+                  safe_dict(namespaces)\
+                  .items() if not k in safe_dict(new_ns).keys()]
+
+        for resource in self.get_resources():
+            # TODO: better idea how to do this?
+            # __dict__ converts rdflib.urirefs to strings -->
+            # converts back to uriref
+            # {'foaf': 'http:/....', ...}
+            namespace_dict = dict(self._graph.namespace_manager.namespaces())
+
+            for triple in resource._tripleserialize_iterator(namespace_dict):
+                graph.add(triple)
+        return graph
 
     def handle_graph(self, follow_uris, handle_owl_imports):
         assert hasattr(self, '_graph')
@@ -949,7 +1018,7 @@ class Origin(Model):
         def add_property_to_resource(origin, subject, predicate, object):
             resource, _created = Resource.objects.get_or_create(uri=subject,
                                                                 origin=self)
-            resource.add_property(predicate, object)
+            resource._add_property(predicate, object)
 
         if follow_uris:
             follow_uris = [rdflib.URIRef(u) if not\
@@ -988,6 +1057,10 @@ class Origin(Model):
 
         self.handled = True
 
+    def has_unsaved_changes(self):
+        if any(resource._has_changes for resource in self.get_resources()\
+               if hasattr(resource, '_has_changes')): return True
+        return False
 
     def PUT(self):
         assert hasattr(self, "_graph")
@@ -995,50 +1068,20 @@ class Origin(Model):
             logging.error("Nothing to PUT for %s!" % self.uri)
             return
         self.backend.PUT(graph=self.graph())
-        # TODO return?
-
-
-    def graph(self):
-        """Processes every Resource and Property related to 'self' and
-        creates rdflib.ConjunctiveGraph because rdflib.Graph does not allow
-        parsing plugins
-        """
-        # TODO: rdflib.graph() ?
-        g = rdflib.graph.ConjunctiveGraph(identifier=self.uri)
-
-        if not hasattr(self, '_graph'):
-            if len(self.errors) == 0:
-                self.GET() # TODO: test for recursion?
-            else:
-                logging.error("Origin %s has Errors --> can't process .graph()"
-                    % self.uri)
-                return g
-
-        # TODO: find a better way to do this
-        # Problems:
-        #  1) namespace bindings are not really necessary to validate
-        #     isomorphic graphs but the resulting graph is is different
-        #     if they miss
-        #  2) doesn't detect duplicate definitions of namespaces
-        namespaces = dict(self._graph.namespace_manager\
-                            .namespaces())
-        for prefix, namespace in safe_dict(namespaces).items():
-            g.bind(prefix=prefix, namespace=namespace)
-        new_ns = dict(g.namespace_manager.namespaces())
-
-        assert namespaces == new_ns, [(k, v) for k, v in
-                  safe_dict(namespaces)\
-                  .items() if  not k in safe_dict(new_ns).keys()]
-
-        for resource in self.get_resources():
-            for triple in resource.tripleserialize_iterator():
-                g.add(triple)
-
-        return g
-
-    def get_resources(self):
-        return Resource.objects.filter(_origin=self)
+        # TODO OK
 
+def check_shortcut_consistency():
+    """Checks every known Origin for inconsistent namespacemappings"""
+    global_namespace_dict = {}
+    for origin in Origin.objects.all():
+        if hasattr(origin, "_graph"):
+            for k, v in dict(origin._graph.namespace_manager\
+                        .namespaces()).items():
+                print k,v
+                if k in global_namespace_dict:
+                    assert global_namespace_dict[k] == v
+                else:
+                    global_namespace_dict[k] = v
 
 # TODO: this is just for convenience --> remove once stable
 def my_graph_diff(graph1, graph2):
